import {Component, OnDestroy, OnInit} from '@angular/core';
import {Subscription} from "rxjs";
import {debounceTime, distinctUntilChanged, map} from "rxjs/operators";
import {TilesService} from "../services/tiles.service";
import {Tile} from "../model/tile";
import {FormControl, FormGroup} from "@angular/forms";
import {AlertService} from "../services/alert.service";
import {faSyncAlt} from "@fortawesome/free-solid-svg-icons";
import {FiltersService} from "../services/filters.service";
import {verticleSlide} from "../animations";

@Component({
  selector: 'app-tiles',
  templateUrl: './tiles.component.html',
  styleUrls: ['./tiles.component.scss'],
  animations: [verticleSlide]
})
export class TilesComponent implements OnInit, OnDestroy {

  public epoch: number = 1;
  public tiles$: Subscription;
  public tiles: Array<Tile>;
  public pattern: string = "";

  public formGroup: FormGroup;

  public faSyncAlt = faSyncAlt;

  private readonly filters$: Subscription;
  public filters: any;
  public sortColumns: Array<string>;
  public sortDirections: Array<string>;

  public showSummary = false;

  constructor(
    private tileService: TilesService,
    private alertService: AlertService,
    private filtersService: FiltersService) {
    this.sortColumns = this.filtersService.getFilter('TILE_SORT');
    this.sortDirections = this.filtersService.getFilter('SORT_DIRECTION');

    this.filters$ = this.filtersService.currentSettings$.subscribe((filters: object) => {
      this.filters = filters;
      this.getMinitilesForEpoch();
    });
  }

  ngOnInit() {
    this.formGroup = new FormGroup({
      pattern: new FormControl()
    });

    this.formGroup.get('pattern').valueChanges.pipe(
      debounceTime(400),
      distinctUntilChanged(),
      map(results => results)
    ).subscribe((val: string) => {
      this.pattern = val;
      this.getMinitilesForEpoch();
    });

    this.getMinitilesForEpoch();
  }

  setEpoch(epoch: number) {
    this.epoch = epoch;
    this.getMinitilesForEpoch();
  }

  setSortColumn(column: string): void {
    this.filtersService.setCurrentSetting('TILE_SORT', column);
  }

  setSortDirection(direction: string): void {
    this.filtersService.setCurrentSetting('SORT_DIRECTION', direction);
  }

  getPrettyName(name: string): string {
    return FiltersService.prettyName(name);
  }

  getEpochName(epoch: number): string {
    switch (epoch) {
      case -1:
        return 'Tests';
      case 0:
        return 'Pilot';
      default:
        return 'Epoch ' + this.epoch.toString();
    }
  }

  updateSummary(): void {
    this.alertService.info('Updating tile summary');
    this.tileService.updateTileSummary(this.epoch).subscribe(() => {
      this.getMinitilesForEpoch();
    }, error => {
      this.alertService.error('Tile summary update failed');
    });
  }

  getMinitilesForEpoch() {
    this.alertService.info('Getting Tiles');
    this.tiles = null;
    this.tiles$ = this.tileService.getTilesForEpoch(this.pattern, this.epoch).subscribe((mt: Array<Tile>) => {
      if (mt && mt.length > 0) {
        this.alertService.success('Tiles retrieved');
        this.tiles = mt;
      } else {
        this.alertService.error('Tiles could not be retrieved');
      }
    });

  }

  ngOnDestroy(): void {
    if (this.tiles$) {
      this.tiles$.unsubscribe();
    }
    if (this.filters$) {
      this.filters$.unsubscribe();
    }
  }

}
